$!******************************************************************************
$!
$!  N E W S S K I M :   Perform regular maintenance tasks for the Usenet News
$!
$!  >>>>>>    MUST RUN from    N E W S M G R    account    <<<<<<
$!
$!  Distribution:  NEWS_ROOT:NEWSSKIM.COM
$!
$!------------------------------------------------------------------------------
$!
$!  Author:	Jonathan Ridler,
$!		University Computing Services,
$!		The University of Melbourne .
$!  Date:	23-Mar-1989
$!  Origin:	Derived from a similar procedure by Geoff Huston.
$!
$!  ** NOTE **  This procedure contains SITE-SPECIFIC code which MUST be
$!		modified for each site using this procedure.  Specific
$!		subroutines containing this code are DO_POST_NEWS and
$!		DO_CONVERT_DB.  Other code may be modified as desired to
$!		meet site-specific requirements.
$!
$!	***	This procedure uses VMS Version 5 DCL constructs,         ***
$!	***	and is appropriate for VMS NEWS Version 5.8               ***
$!
$!      V5.9    15-May-1989     GIH
$!        - Remove convert/reclaim as it is functionally useless!
$!******************************************************************************
$
$ show time				!  Just for info.
$ show logical SYS$NODE			!  Ditto.
$ set output_rate = :00:20		!  Allow a close watch to be kept
$
$ set noon
$ set control = (Y,T)
$
$ Run_Times = "01,04,07,10,12,15,18,21"	!  Times when to submit during the day.
$ Convert_Day  = "Sunday"		!  Day  to convert database files.
$ Convert_Hour = "01"			!  Hour to convert database files.
$
$ Batch_Q = "P3$BATCH"			!  Batch queue in which NEWSSKIM runs.
$					!  ** SITE-SPECIFIC **
$
$ Add_News	= 0
$ Skim_News	= 0
$ Post_News	= 0
$ Convert_Db	= 0
$ Special	= 0
$ Resubmit	= 1			!  Default = Resubmit.
$
$ verify = 'f$verify(0)'
$
$ ws := write SYS$OUTPUT		!  Can use REQUEST if prefered.
$
$ Node_Search = 1			!  Search stream id for Post_News
$ File_Search = 2			!  Search stream id for Post_News
$ NNTP_Proto_Search = 3			!  Search stream id for NNTP_News
$ NNTP_Node_Search  = 4			!  Search stream id for NNTP_News
$
$ Now   = f$time ()
$ Hour  = f$extract (12,2,Now)
$ Today = f$cvtime (Now,,"WEEKDAY")
$
$ set on
$ on error  then  goto  ABORT
$
$ if p1 .eqs. ""  then  goto  'Hour'	!  A normally scheduled run.
$
$!..............................................................................
$!  Something special as a one-off run - handle it separately as specified in
$!  the parameter list.
$
$ gosub  Parse_Params
$ goto  ZOOM
$
$!..............................................................................
$!  Scheduled run - do tasks as indicated below by hour.
$
$01:
$	gosub ALL_TASKS
$	goto  ZOOM
$
$04:
$07:
$10:
$12:
$15:
$18:
$21:
$	gosub NORMAL_TASKS
$	goto  ZOOM
$
$00:
$02:
$03:
$05:
$06:
$08:
$09:
$11:
$13:
$14:
$16:
$17:
$19:
$20:
$22:
$23:
$	goto  ZOOM
$
$!..............................................................................
$ZOOM:
$
$ on error  then  goto  ABORT
$
$ if Special	then  gosub  Do_Special		! Black hole - no return.
$
$ if Skim_news  then  gosub  Do_Skim_News	! Keep db trim: Skim before Add
$ if Add_news   then  gosub  Do_Add_News
$ if Post_news  then  gosub  Do_Post_News
$ if Post_news  then  gosub  Do_NNTP_News
$ if Convert_Db then  gosub  Do_Convert_Db
$
$ if Resubmit   then  gosub  Resubmit_Self
$
$ if verify then set verify
$ EXIT
$
$!==============================================================================
$! Sub-Routines follow here:
$!------------------------------------------------------------------------------
$!  Set flags for ALL TASKS - only convert the database once a week.
$
$ALL_TASKS:
$
$ Add_News	= 1
$ Skim_News	= 1
$ Post_News	= 1
$ if (Today .eqs. Convert_Day) .and. -
     (Hour .eqs. Convert_Hour)  then  Convert_Db = 1
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Set flags for NORMAL TASKS - usually ones that should happen more than once
$!  a day.
$
$NORMAL_TASKS:
$
$ Add_News	= 1
$ Post_News	= 1
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Parse any parameters that exist.  Allow only valid functions as defined
$!  in this DCL procedure.  Tasks must be specified one per parameter, e.g.
$!   P1 = ADD_NEWS  P2 = POST_NEWS
$!  Any valid abbreviation of 2 characters or greater may be specified.
$
$PARSE_PARAMS:
$
$ Resubmit = 0		!  Must specifically request this when special run.
$ Param = 0
$
$GET_PARAM:
$ Param = Param + 1
$ Cmd = P'Param'
$ if Cmd .eqs. ""  then  RETURN
$ Cmd = f$edit (Cmd,"COLLAPSE,UNCOMMENT,UPCASE")
$ if f$length (Cmd) .ge. 2  then  goto  CMD_LEN_OK
$ ws "NEWSSKIM Error: parameters must be >= 2 characters (parameter P''Param')"
$ goto  ABORT
$
$CMD_LEN_OK:
$ Cmd = f$extract (0,2,Cmd)				! First 2 chars only.
$ if Cmd .nes. "SP"  then  goto  SET_CMD		! "SPECIAL"
$
$!  Wants a special one-off user-specified run ...
$ if (Param .eq. 1) .and. (P2 .nes. "")  then  goto  SET_SPECIAL
$ if Param .ne. 1  then  ws "NEWSSKIM Error: SPECIAL parameter must be P1."
$ if P2 .eqs. ""   then  ws "NEWSSKIM Error: SPECIAL parameter P2 is missing."
$ goto ABORT
$
$SET_SPECIAL:
$ Special = 1
$ RETURN
$
$SET_CMD:
$ if Cmd .eqs. "AD"  then  Add_News   = 1
$ if Cmd .eqs. "PO"  then  Post_News  = 1
$ if Cmd .eqs. "SK"  then  Skim_News  = 1
$ if Cmd .eqs. "CO"  then  Convert_Db = 1
$ if Cmd .eqs. "RE"  then  Resubmit   = 1
$ if Cmd .eqs. "AL"  then  gosub All_Tasks
$ goto  GET_PARAM
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Extract MAILed items; Add any extracted MAIL files to NEWS and any other
$!  batch files that may have been created by other means.
$
$DO_ADD_NEWS:
$
$ ws "Extracting mailed News items ..."
$ mail
set folder newmail
move/all temp
y
set folder temp
extract /all /noheaders NEWS_MANAGER:NEWMAIL.MAIL
delete /all
exit
$
$ if f$search ("NEWS_MANAGER:NEWMAIL.MAIL") .nes. ""  then  goto  ADD_MAIL
$ if f$search ("NEWS_MANAGER:NEWS.BATCH") .nes. ""    then  goto  ADD_BATCH
$ ws "No news items to add ... skipping ADD to News."
$ RETURN
$
$ADD_MAIL:
$ ws "Adding mailed and forwarded items to news database ..."
$ news /noscreen
Add File NEWS_MANAGER:NEWMAIL.MAIL;*
exit
$
$ set noon
$ rename /log NEWS_MANAGER:NEWMAIL.MAIL;* NEWS_MANAGER:OLDMAIL.MAIL;
$ purge /keep=14 NEWS_MANAGER:OLDMAIL.MAIL
$ set on
$
$ if f$search ("NEWS_MANAGER:NEWS.BATCH") .eqs. ""  then  RETURN
$
$ADD_BATCH:
$ ws "Adding batched items to news database ..."
$ news /noscreen
Add File /delete NEWS_MANAGER:NEWS.BATCH;*
exit
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Skim the NEWS database.  ==> skim /items /newsgroups /files /served
$!  News items in comp.os.vms are archived before the main Skim.
$
$DO_SKIM_NEWS:
$
$ ws "Skimming the news database ..."
$ news /noscreen
Skim/nofile/nonews/noserv/item/arch=news_root:archive.comp_os_vms comp.os.vms
Skim
exit
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Send any News items to DECNet nodes by the appropriate method.
$!  Search for all directories of the form NEWS_MANAGER:POST_<node>.DIR .
$!  Send all batch files in the node-specific directory to the remote node.
$!  Delete each batch file in the directory after successful transfer.
$!  Changes may need to be made to this sub-routine, depending on the
$!  mechanism for interchange of news items between nodes.
$!
$!  ** NOTE **  This subroutine contains SITE-SPECIFIC code.
$
$DO_POST_NEWS:
$
$ ws "Checking for (and forwarding) news to other sites via MAIL ..."
$ Xfer_Failure = 0
$ Failure_Limit = 3
$
$DIR_SEARCH:
$ on error  then  goto  ABORT
$
$ if Xfer_Failure .eq. 0  then  goto  LOOKUP
$ mail SYS$INPUT usenet -
 /subject="Automatic NEWSSKIM alarm: MAIL XFER_FAILURE to ''Node' (''f$time()')"
The News_Skim job failed to transfer some news via MAIL - please investigate.
$
$LOOKUP:
$ Post_Dir = f$search ("NEWS_MANAGER:POST_*.DIR",Node_Search)
$ if Post_Dir .eqs. ""  then  RETURN
$ Xfer_Failure = 0
$
$ Node = f$parse (Post_Dir,,,"NAME","SYNTAX_ONLY") - "POST_"
$ News_Agent := ""
$
$!  *** SITE-SPECIFIC *** SECTION (1 of 2) ...
$
$ if Node .eqs. "GHOST"    then  goto  DIR_SEARCH	!  Just a pretendy node.
$ if Node .eqs. "LICRA"    then  News_Agent := LICRA::NEWSMGR
$ if Node .eqs. "MURDU"    then  News_Agent := """in%""""rnews@murdu"""""""
$
$!  ** END ** Site-Specific Section.
$
$ if News_Agent .eqs. ""  then  goto  DIR_SEARCH	!  Unknown node.
$
$ Post_Files = "NEWS_MANAGER_DEV:[POST_" + Node + "]*.*;*"
$ ws "Checking for News to send to node ''Node' ..."
$
$FILE_SEARCH:
$ Batch_File = f$search (Post_Files,File_Search)
$ if Batch_File .eqs. ""  then  goto  DIR_SEARCH
$
$ on error  then  goto  XFER_ERROR_MAIL
$
$!  *** SITE-SPECIFIC *** SECTION (2 of 2): Change "UCSVC" to local nodename.
$
$ ws "Mailing to ''Node': ''Batch_File'"
$ mail 'Batch_File' 'News_Agent' /subject="Usenet News Item(s) from UCSVC"
$ delete 'Batch_File'
$ goto  FILE_SEARCH
$
$XFER_ERROR_MAIL:
$ Xfer_Failure = Xfer_Failure + 1
$ if Xfer_Failure .ge. Failure_Limit  then  goto  DIR_SEARCH
$ goto  FILE_SEARCH
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Send News items using the NNTP protocols.  Search for all directories of
$!  the form  NEWS_MANAGER:NNTP_<protocol>.DIR  where <protocol> is one of
$!  "DECNET", "CMUTCP", "WINTCP", etc.  These directories will contain files
$!  of the form  <node>.IDS  where <node> indicates the node to feed.
$
$DO_NNTP_NEWS:
$
$ ws "Checking for (and forwarding) news to other sites via NNTP ..."
$ nntp_feed := $NEWS_MANAGER:NNTP_FEED
$
$DIR_SEARCH_PROTOCOLS:
$ on error  then  goto  ABORT
$ NNTP_Dir = f$search ("NEWS_MANAGER:NNTP_*.DIR",NNTP_Proto_Search)
$ if NNTP_Dir .eqs. ""  then  RETURN
$
$ Protocol = f$parse (NNTP_Dir,,,"NAME","SYNTAX_ONLY") - "NNTP_"
$ NNTP_Files = "NEWS_MANAGER_DEV:[NNTP_" + Protocol + "]*.IDS;*"
$
$NODE_SEARCH:
$ Ihave_File = f$search (NNTP_Files,NNTP_Node_Search)
$ if Ihave_File .eqs. ""  then  goto  DIR_SEARCH_PROTOCOLS
$
$ Node = f$parse (Ihave_File,,,"NAME","SYNTAX_ONLY")
$
$ on error  then  goto  XFER_FAILURE_NNTP
$ ws "NNTP_FEED: Connecting to node ''Node' (using protocol ''Protocol') ..."
$ nntp_feed 'Node' 'Protocol' 'Ihave_File' NEWS_MANAGER:NNTP_FEED.LOG 'Node'.LOG
$ goto NODE_SEARCH
$
$XFER_ERROR_NNTP:
$ mail SYS$INPUT usenet -
 /subject="Automatic NEWSSKIM alarm: NNTP XFER_FAILURE to ''Node' (''f$time()')"
The News_Skim job failed to transfer some news via NNTP - please investigate.
$
$ goto  NODE_SEARCH
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Convert the News database files to optimize their structure.
$!  Also, compress the NEWSMGR MAIL file to (dramatically) improve efficiency.
$
$DO_CONVERT_DB:
$
$ ws "Converting news database ..."
$ on error  then  goto  DROP_OUT
$
$ convert NEWS_ROOT:NEWS.GROUPS NEWS_ROOT:NEWS.GROUPS
$ purge /log NEWS_ROOT:NEWS.GROUPS
$ set file /log /global_buffer=4 NEWS_ROOT:NEWS.GROUPS  ! Improve performance
$
$ convert NEWS_ROOT:NEWS.ITEMS NEWS_ROOT:NEWS.ITEMS
$ purge /log NEWS_ROOT:NEWS.ITEMS
$ set file /log /global_buffer=12 NEWS_ROOT:NEWS.ITEMS  ! Improve performance
$
$ convert NEWS_ROOT:HISTORY.V60 NEWS_ROOT:HISTORY.V60
$ purge /log NEWS_ROOT:HISTORY.V60
$
$ ws "Compressing the NEWSMGR MAIL file ..."
$ Max_Tries = 3
$ Tries = 0
$
$COMPRESS_MAIL:
$ mail
compress
exit
$
$ if .not. $status			! MAIL file may be locked.
$ then
$	Tries = Tries + 1
$	if Tries .lt. Max_Tries
$	then
$		ws "Re-trying MAIL Compression ..."
$		wait 00:00:30
$		goto  COMPRESS_MAIL
$	else
$		ws "** FAILED to compress MAIL file **"
$	endif
$ endif
$
$ purge /log SYS_2:[NEWSMGR.MAIL]MAIL.OLD	! *** SITE-SPECIFIC name ***
$
$ on error  then  goto  ABORT
$ RETURN
$
$DROP_OUT:
$ on error  then  goto  ABORT
$ mail SYS$INPUT usenet -
  /subject="Automatic NEWSSKIM alarm: CONVERT_DB FAILED (''f$time()')"
The News_Skim job failed to convert the database - please investigate.
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Re-Submit this job at the appropriate time.  Allow a few minutes after the
$!  hour to ensure that any clock discrepencies in the cluster are irrelevant.
$
$RESUBMIT_SELF:
$
$ Hour = f$extract (12,2,f$time())		!  Use current time.
$ Cnt  = 0
$
$GET_NEXT:
$ When = f$element (Cnt,",",Run_Times)
$ if When .nes. ","  then  goto  COMPARE_HOUR
$ When = f$element (0,",",Run_Times)		!  Use first run for tomorrow.
$ goto  SCHEDULER
$
$COMPARE_HOUR:
$ Cnt = Cnt + 1
$ if When .les. Hour  then  goto  GET_NEXT	!  Zip along until ok.
$
$SCHEDULER:
$ if When .gts. Hour  then  When = When + ":10"
$ if When .les. Hour  then  When = "Tomorrow+" + When + ":10"
$
$ ws "Re-submitting NEWSSKIM ..."
$ submit NEWS_ROOT:NEWSSKIM.COM -
	/noprint -
	/notify -
	/keep -
	/queue = 'Batch_Q' -
	/cputime = 06:00:00 -
	/log = NEWS_ROOT:NEWSSKIM.LOG -
	/name = "News_Skim" -
	/after = "''When'"
$
$ set noon
$ purge /keep=18 NEWS_ROOT:NEWSSKIM.LOG
$ set on
$
$ RETURN
$
$!------------------------------------------------------------------------------
$!  Special run - use parameters.
$
$DO_SPECIAL:
$
$ ws "SPECIAL RUN."
$ open /write  cmd_file  sys$scratch:temp_input.dat
$ write  cmd_file  "''p2'"				!  P2 must be there.
$ if p3 .nes. ""  then  write  cmd_file  "''p3'"
$ if p4 .nes. ""  then  write  cmd_file  "''p4'"
$ if p5 .nes. ""  then  write  cmd_file  "''p5'"
$ if p6 .nes. ""  then  write  cmd_file  "''p6'"
$ if p7 .nes. ""  then  write  cmd_file  "''p7'"
$ if p8 .nes. ""  then  write  cmd_file  "''p8'"
$ write  cmd_file  "exit"
$ close cmd_file
$ assign /user_mode sys$scratch:temp_input.dat SYS$INPUT
$ news /noscreen
$ delete sys$scratch:temp_input.dat;*
$
$ EXIT
$
$!------------------------------------------------------------------------------
$!  Come here to die unnaturally.  Notify USENET account (*NOT* NEWSMGR!!).
$
$ABORT:
$
$ ws "######  NEWSSKIM terminated ON ABORT  ######"
$
$ mail SYS$INPUT usenet -
  /subject="Automatic NEWSSKIM alarm: ABORTED (''f$time()')"
The News_Skim job ABORTED - please investigate.
$
$ EXIT
